
use builtin;
use str;

set edit:completion:arg-completer[rustup] = {|@words|
    fn spaces {|n|
        builtin:repeat $n ' ' | str:join ''
    }
    fn cand {|text desc|
        edit:complex-candidate $text &display=$text' '(spaces (- 14 (wcswidth $text)))$desc
    }
    var command = 'rustup'
    for word $words[1..-1] {
        if (str:has-prefix $word '-') {
            break
        }
        set command = $command';'$word
    }
    var completions = [
        &'rustup'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
            cand -V 'Print version information'
            cand --version 'Print version information'
            cand -v 'Enable verbose output'
            cand --verbose 'Enable verbose output'
            cand -q 'Disable progress output'
            cand --quiet 'Disable progress output'
            cand dump-testament 'Dump information about the build'
            cand show 'Show the active and installed toolchains or profiles'
            cand install 'Update Rust toolchains'
            cand uninstall 'Uninstall Rust toolchains'
            cand update 'Update Rust toolchains and rustup'
            cand check 'Check for updates to Rust toolchains and rustup'
            cand default 'Set the default toolchain'
            cand toolchain 'Modify or query the installed toolchains'
            cand target 'Modify a toolchain''s supported targets'
            cand component 'Modify a toolchain''s installed components'
            cand override 'Modify directory toolchain overrides'
            cand run 'Run a command with an environment configured for a given toolchain'
            cand which 'Display which binary will be run for a given command'
            cand doc 'Open the documentation for the current toolchain'
            cand man 'View the man page for a given command'
            cand self 'Modify the rustup installation'
            cand set 'Alter rustup settings'
            cand completions 'Generate tab-completion scripts for your shell'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;dump-testament'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;show'= {
            cand -v 'Enable verbose output with rustc information for all installed toolchains'
            cand --verbose 'Enable verbose output with rustc information for all installed toolchains'
            cand -h 'Print help information'
            cand --help 'Print help information'
            cand active-toolchain 'Show the active toolchain'
            cand home 'Display the computed value of RUSTUP_HOME'
            cand profile 'Show the current profile'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;show;active-toolchain'= {
            cand -v 'Enable verbose output with rustc information'
            cand --verbose 'Enable verbose output with rustc information'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;show;home'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;show;profile'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;show;help'= {
        }
        &'rustup;install'= {
            cand --profile 'profile'
            cand --no-self-update 'Don''t perform self-update when running the `rustup install` command'
            cand --force 'Force an update, even if some components are missing'
            cand --force-non-host 'Install toolchains that require an emulator. See https://github.com/rust-lang/rustup/wiki/Non-host-toolchains'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;uninstall'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;update'= {
            cand --no-self-update 'Don''t perform self update when running the `rustup update` command'
            cand --force 'Force an update, even if some components are missing'
            cand --force-non-host 'Install toolchains that require an emulator. See https://github.com/rust-lang/rustup/wiki/Non-host-toolchains'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;check'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;default'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;toolchain'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
            cand list 'List installed toolchains'
            cand install 'Install or update a given toolchain'
            cand uninstall 'Uninstall a toolchain'
            cand link 'Create a custom toolchain by symlinking to a directory'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;toolchain;list'= {
            cand -v 'Enable verbose output with toolchain information'
            cand --verbose 'Enable verbose output with toolchain information'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;toolchain;install'= {
            cand --profile 'profile'
            cand -c 'Add specific components on installation'
            cand --component 'Add specific components on installation'
            cand -t 'Add specific targets on installation'
            cand --target 'Add specific targets on installation'
            cand --no-self-update 'Don''t perform self update when running the`rustup toolchain install` command'
            cand --force 'Force an update, even if some components are missing'
            cand --allow-downgrade 'Allow rustup to downgrade the toolchain to satisfy your component choice'
            cand --force-non-host 'Install toolchains that require an emulator. See https://github.com/rust-lang/rustup/wiki/Non-host-toolchains'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;toolchain;uninstall'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;toolchain;link'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;toolchain;help'= {
        }
        &'rustup;target'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
            cand list 'List installed and available targets'
            cand add 'Add a target to a Rust toolchain'
            cand remove 'Remove a target from a Rust toolchain'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;target;list'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --installed 'List only installed targets'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;target;add'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;target;remove'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;target;help'= {
        }
        &'rustup;component'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
            cand list 'List installed and available components'
            cand add 'Add a component to a Rust toolchain'
            cand remove 'Remove a component from a Rust toolchain'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;component;list'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --installed 'List only installed components'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;component;add'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --target 'target'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;component;remove'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --target 'target'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;component;help'= {
        }
        &'rustup;override'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
            cand list 'List directory toolchain overrides'
            cand set 'Set the override toolchain for a directory'
            cand unset 'Remove the override toolchain for a directory'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;override;list'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;override;set'= {
            cand --path 'Path to the directory'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;override;unset'= {
            cand --path 'Path to the directory'
            cand --nonexistent 'Remove override toolchain for all nonexistent directories'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;override;help'= {
        }
        &'rustup;run'= {
            cand --install 'Install the requested toolchain if needed'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;which'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;doc'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand --path 'Only print the path to the documentation'
            cand --alloc 'The Rust core allocation and collections library'
            cand --book 'The Rust Programming Language book'
            cand --cargo 'The Cargo Book'
            cand --core 'The Rust Core Library'
            cand --edition-guide 'The Rust Edition Guide'
            cand --nomicon 'The Dark Arts of Advanced and Unsafe Rust Programming'
            cand --proc_macro 'A support library for macro authors when defining new macros'
            cand --reference 'The Rust Reference'
            cand --rust-by-example 'A collection of runnable examples that illustrate various Rust concepts and standard libraries'
            cand --rustc 'The compiler for the Rust programming language'
            cand --rustdoc 'Documentation generator for Rust projects'
            cand --std 'Standard library API documentation'
            cand --test 'Support code for rustc''s built in unit-test and micro-benchmarking framework'
            cand --unstable-book 'The Unstable Book'
            cand --embedded-book 'The Embedded Rust Book'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;man'= {
            cand --toolchain 'Toolchain name, such as ''stable'', ''nightly'', or ''1.8.0''. For more information see `rustup help toolchain`'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;self'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
            cand update 'Download and install updates to rustup'
            cand uninstall 'Uninstall rustup.'
            cand upgrade-data 'Upgrade the internal data format.'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;self;update'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;self;uninstall'= {
            cand -y 'y'
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;self;upgrade-data'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;self;help'= {
        }
        &'rustup;set'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
            cand default-host 'The triple used to identify toolchains when not specified'
            cand profile 'The default components installed'
            cand auto-self-update 'The rustup auto self update mode'
            cand help 'Print this message or the help of the given subcommand(s)'
        }
        &'rustup;set;default-host'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;set;profile'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;set;auto-self-update'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;set;help'= {
        }
        &'rustup;completions'= {
            cand -h 'Print help information'
            cand --help 'Print help information'
        }
        &'rustup;help'= {
        }
    ]
    $completions[$command]
}
